/**
 * Equipment related functions for the Carnage Blender extension
 */

// Extract armor stats for a specific minion tab
const extractArmorStatsForTab = async (minionDiv) => {
  if (!window.CBState.loadedItems) {
    await window.CBStorage.loadItemsData();
  }
  
  let nameElement = minionDiv.querySelector('h2');
  if (!nameElement) {
    return false;
  }
  const minionName = nameElement.textContent.trim();
  const stats = {
    minionName: minionName,
    baseAC: 0,
    enchantmentAC: 0,
    totalAC: 0,
    physicalReduction: 0,
    magicReduction: 0,
    equippedItems: []
  };

  // Verbesserte Item-Erkennung - jetzt nur Items innerhalb des aktuellen minion divs
  const items = minionDiv.querySelectorAll('tr');
  items.forEach(item => {  
    const itemCell = item.querySelector('td');
    if (!itemCell) return;
    
    const itemText = itemCell.textContent.trim();
    // Überspringe leere Slots oder "None" Einträge
    if (itemText === 'None' || !itemText) return;
    
    // Prüfe ob wir im "unequipped" Bereich sind und überspringe alle weiteren Items
    const unequippedDiv = itemCell.closest('#unequipped');
    if (unequippedDiv) return;

    // Erweiterte Regex für [X] und (+Y) Erkennung
    const itemMatch = itemText.match(/^(?:A |An |The )?([^[]+?)(?:\s*\[(\d+)(?:x\d+)?\])?\s*(?:\((\+\d+)\))?$/);
    
    if (itemMatch) {
      const [_, baseName, itemLevel, enchantment] = itemMatch;
      const level = itemLevel ? parseInt(itemLevel) : 0;
      const enchantValue = enchantment ? parseInt(enchantment.replace(/\+/g, '')) : 0;
      
      // Finde mögliche Matches (case-insensitive)
      const match = window.CBUtils.searchItemInJSON(baseName, window.CBState.loadedItems);
      if (!match) {
        return;
      }
      const jsonMatch = match.data;
      // Erstelle das Item-Objekt - jetzt für alle Items
      const itemObject = {
        originalName: baseName.trim(),
        normalizedName: baseName.trim(),
        level,
        enchantment: enchantValue,
        displayName: itemText,
        foundInJson: !!jsonMatch
      };

      // Füge zusätzliche Informationen hinzu, wenn ein JSON-Match gefunden wurde
      if (jsonMatch) {
        itemObject.type = jsonMatch.type;
        itemObject.hands = jsonMatch.hands;
        itemObject.jsonMatch = jsonMatch;
      } else {
        itemObject.type = 'unknown';
      }
      stats.equippedItems.push(itemObject);
    }
  });

  // Prüfe zuerst auf Schild
  const hasShield = stats.equippedItems.some(item => 
    (item.jsonMatch?.type === 'armor' && item.jsonMatch?.slot === 'shield')
  );
  // Wenn kein Schild vorhanden ist, können wir direkt zurückkehren
  if (hasShield) {
    const weapons = stats.equippedItems.filter(item => {
      return item.jsonMatch?.type === 'weapon';
    });
  
    const weapon = weapons[0]; // Nimm die erste gefundene Waffe
  
    // Prüfe auf 2H-Waffe basierend auf dem hands Attribut in items.json
    const is2HWeapon = weapon && weapon.jsonMatch?.hands === 2;
  
    if (is2HWeapon) {
      // Füge -12% DX Malus hinzu
      if (!stats.calculatedEffects) {
        stats.calculatedEffects = {};
      }
      if (!stats.calculatedEffects.statModifiers) {
        stats.calculatedEffects.statModifiers = {};
      }
      stats.calculatedEffects.statModifiers.dx_shield_penalty = -0.12;
    }
  }
  // Berechne calculatedEffects für Rüstungsteile
  stats.equippedItems.forEach(item => {
    if (!item.calculatedEffects) {
      item.calculatedEffects = {};
    }

    if (item.jsonMatch?.type === 'armor') {
      // Basis-AC aus den Effekten
      const baseAC = item.jsonMatch.effects?.base_ac || 0;
      
      // Enchanted AC wenn vorhanden
      if (item.jsonMatch.enchanted_ac) {
        item.calculatedEffects.enchantedAC = item.enchantment;
        item.calculatedEffects.totalAC = baseAC + item.enchantment;
      } else {
        item.calculatedEffects.totalAC = baseAC;
      }

      // Stat-Modifikatoren pro + berechnen
      if (item.jsonMatch.effects?.effects_per_plus) {
        item.calculatedEffects.statModifiers = {};
        
        Object.entries(item.jsonMatch.effects.effects_per_plus).forEach(([stat, value]) => {
          item.calculatedEffects.statModifiers[stat] = value * item.enchantment;
        });
      }
    }
  });

  // Berechne Gesamt-AC-Werte
  let totalBaseAC = 0;
  let totalEnchantedAC = 0;

  stats.equippedItems.forEach(item => {
    const itemName = item.displayName;
    // Extrahiere Base AC aus [X] Pattern
    const baseACMatch = itemName.match(/\[(\d+)\]/);
    const baseAC = baseACMatch ? parseInt(baseACMatch[1]) : 0;
    
    // Extrahiere Enchanted AC aus (+Y) Pattern
    const enchantedACMatch = itemName.match(/\(\+(\d+)\)/);
    const enchantedAC = enchantedACMatch ? parseInt(enchantedACMatch[1]) : 0;
    
    totalBaseAC += baseAC;
    totalEnchantedAC += enchantedAC;
  });

  // Aktualisiere die Stats
  stats.baseAC = totalBaseAC;
  stats.enchantmentAC = totalEnchantedAC;
  stats.totalAC = totalBaseAC + totalEnchantedAC;
  stats.damageReduction = {
    physical: `${Math.min(90, Math.round((totalBaseAC + totalEnchantedAC) * 0.167))}%`,
    magical: `${Math.min(75, Math.round(totalEnchantedAC * 0.167))}%`
  };
  return stats;
};

// Extract armor stats for all minions
const extractAllArmorStats = async (doc) => {
  const allStats = {};
  // Suche alle DIVs, die als Container der einzelnen Minion-Tabs dienen
  const tabContainers = doc.querySelectorAll('div.tabs-container');
  
  // Für jeden gefundenen Tab rufen wir die Unterfunktion auf
  for (const tabContainer of tabContainers) {
    const tabStats = await extractArmorStatsForTab(tabContainer);
    if (!tabStats) {
      continue;
    }
    
    const minionName = tabStats["minionName"];
    allStats[minionName] = tabStats;
  }
  return allStats;
};

// Export equip functions
window.CBEquip = {
  extractArmorStatsForTab,
  extractAllArmorStats
};