/**
 * Utility functions for the Carnage Blender extension
 */

// Stat abbreviations for display
const statAbbrev = {
  'strength': 'ST',
  'hitpoints': 'HP', 
  'dexterity': 'DX',
  'enchant_offense': 'EO',
  'enchant_defense': 'ED',
  'damage_defense': 'DD',
  'skill': 'Skill',
  'minion_direct_damage': 'DD',
  'minion_hit': 'PTH Bonus',
  'minion_strength': 'ST',
  'magic': 'Magic Reduction',
  'physical_absorption': 'Phys. Absorption', 
  'enemy_hit_penalty': 'Enemy Hit Penalty',
  'leadership': 'Leadership',
  'direct_damage_resistance': 'DD Resistance',
  'ac': 'AC',
  'unarmed_combat': 'UC',
  'direct_damage': 'DD'
};

// Normalize item names for comparison
const normalizeItemName = (name) => {
  if (!name) return '';
  
  let normalized = name;
  
  // Entferne Artikel am Anfang
  normalized = normalized.replace(/^(a|an|the)\s+/i, '');

  // Entferne "Set of", "Pair of" etc.
  normalized = normalized.replace(/^(set of|pair of)\s+/i, '');

  // Entferne Apostrophe und 's am Ende von Wörtern
  normalized = normalized.replace(/('s|')\s+/g, ' ');

  // Entferne alle Apostrophe, Bindestriche und Leerzeichen
  normalized = normalized.replace(/['-\s]/g, '');

  // Entferne "lvl" und nachfolgende Zahlen
  normalized = normalized.replace(/lvl[\d,]+/i, '');

  // Alles in Kleinbuchstaben
  normalized = normalized.toLowerCase();

  return normalized;
};

// Helper: safely parse number strings
const safeParse = str => {
  if (!str || typeof str !== "string" || str.length === 0) return 0;
  const num = parseFloat(str.replace(/,/g, ""));
  return isNaN(num) ? 0 : num;
};

// Zentrale Hilfsfunktion für Sidebar-Frame
const getSidebarFrame = () => window.parent.frames["sidebar"];

// Parse equipped item text
function parseEquippedItemText(itemText) {
  // Entferne "A ", "An ", "The " etc.
  const cleanText = itemText.replace(/^(A|An|The)\s+/i, '').trim();

  // Erweiterte Regex für [X], (+Y) und lvl Z Erkennung
  // Gruppe 1: Item Name
  // Gruppe 2: Base Level [X]
  // Gruppe 3: Enchantment (+Y)
  // Gruppe 4: Tattoo Level
  const itemRegex = /^(.*?)(?:\s*\[(\d+(?:x\d+)?)\])?\s*(?:\(\+(\d+)\))?\s*(?:lvl\s+([\d,]+))?$/i;
  const match = cleanText.match(itemRegex);
  
  if (!match) {
      return null;
  }

  const [ , rawName, rawBaseLevel, rawEnchant, rawLevel ] = match;
  
  // Entferne zusätzliche Leerzeichen
  const cleanName = rawName.trim();

  // Base-Level in Zahl. Falls "4x2750" => wir picken den vorderen Teil, z.B. 4
  let baseLevel = 0;
  if (rawBaseLevel) {
      const baseLevelMatch = rawBaseLevel.match(/^(\d+)/);
      if (baseLevelMatch) {
          baseLevel = parseInt(baseLevelMatch[1], 10);
      }
  }

  let enchantValue = 0;
  if (rawEnchant) {
      enchantValue = parseInt(rawEnchant, 10);
  }

  let tattooLevel = null;
  if (rawLevel) {
      // Entferne Kommas aus der Zahl
      tattooLevel = parseInt(rawLevel.replace(/,/g, ''), 10);
  }

  return {
      originalText: itemText.trim(),
      name: cleanName,
      normalizedName: normalizeItemName(cleanName),
      baseLevel,
      enchantValue,
      tattooLevel
  };
}

// Search for item in JSON database
function searchItemInJSON(searchName, items) {
  // Normalisiere nur den Suchtext
  const normalizedSearchName = searchName;

  // Exakte Suche im Original
  if (items[searchName]) {
      return {
          key: searchName,
          data: items[searchName]
      };
  }

  // Versuche verschiedene Normalisierungsstufen für die Suche
  let normalized = searchName;
  let matches = Object.entries(items).filter(([key]) => key === normalized);
  if (matches.length > 0) return { key: matches[0][0], data: matches[0][1] };

  // 1. Entferne Artikel am Anfang
  normalized = normalized.replace(/^(a|an|the)\s+/i, '');
  matches = Object.entries(items).filter(([key]) => key === normalized);
  if (matches.length > 0) return { key: matches[0][0], data: matches[0][1] };
  
  // 2. Entferne "Set of", "Pair of" etc.
  normalized = normalized.replace(/^(set of|pair of)\s+/i, '');
  matches = Object.entries(items).filter(([key]) => key === normalized);
  if (matches.length > 0) return { key: matches[0][0], data: matches[0][1] };
  
  // 4. Entferne Apostrophe
  normalized = normalized.replace(/[']/g, '');
  matches = Object.entries(items).filter(([key]) => 
      key.replace(/[']/g, '') === normalized
  );
  if (matches.length > 0) return { key: matches[0][0], data: matches[0][1] };
  

  // 4. Entferne Bindestriche und Leerzeichen
  normalized = normalized.replace(/[-\s]/g, '');
  matches = Object.entries(items).filter(([key]) => 
      key.replace(/[-\s]/g, '') === normalized
  );
  if (matches.length > 0) return { key: matches[0][0], data: matches[0][1] };
  
  // 5. Alles in Kleinbuchstaben
  normalized = normalized.toLowerCase();
  matches = Object.entries(items).filter(([key]) => 
      key.replace(/['-\s]/g, '').toLowerCase() === normalized
  );

  if (matches.length > 0) return { key: matches[0][0], data: matches[0][1] };

  return null;
}

// Calculate AC values
const calculateAC = (baseAC, enchantmentAC, hasSteelSkin = false) => {
  // Basis-Schadensreduktion: baseAC * random(100-200%)
  const baseReduction = baseAC * 1.5; // Durchschnittswert 150%
  
  // Physische Schadensreduktion: 0.167% pro AC-Punkt (base + enchantment)
  // Mit Steel Skin: 0.21% pro AC-Punkt
  const reductionPerPoint = hasSteelSkin ? 0.21 : 0.167;
  const physicalReduction = (baseAC + enchantmentAC) * reductionPerPoint;
  
  // Magische Schadensreduktion: 0.167% pro enchantment AC
  const magicReduction = enchantmentAC * 0.167;

  return {
    baseAC,
    enchantmentAC,
    totalAC: baseAC + enchantmentAC,
    physicalReduction: Math.round(physicalReduction * 100) / 100,
    magicReduction: Math.round(magicReduction * 100) / 100
  };
};

// Calculate item effects
function calculateItemEffects(itemData, baseLevel, enchantValue) {
  const effects = {
      statModifiers: {},
      baseAC: 0,
      enchantedAC: 0,
      totalAC: 0
  };

  // Früher Ausstieg wenn keine jsonMatch.effects vorhanden
  if (!itemData?.jsonMatch?.effects) {
      return effects;
  }

  const itemEffects = itemData.jsonMatch.effects;

  // Base AC aus jsonMatch.effects.base_ac
  effects.baseAC = itemEffects.base_ac || 0;

  // Enchanted AC direkt aus dem enchantment Feld wenn erlaubt
  if (itemEffects.enchanted_ac === true) {
      effects.enchantedAC = itemData.enchantment;
  }

  // Total AC ist die Summe
  effects.totalAC = effects.baseAC + effects.enchantedAC;

  // Stat-Modifikatoren
  if (itemEffects.effects_per_plus) {
      Object.entries(itemEffects.effects_per_plus).forEach(([stat, value]) => {
          effects.statModifiers[stat] = value * itemData.enchantment;
      });
  }

  return effects;
}

// Build equipped item object
function buildEquippedItem(itemText, loadedItems) {
  const parsedItem = parseEquippedItemText(itemText);
  if (!parsedItem) return null;

  // Sammle alle Items
  const allItems = {
      ...(loadedItems.items || {}),
      ...loadedItems
  };

  // Suche direkt mit den Original-Keys
  const match = searchItemInJSON(parsedItem.name, allItems);
  
  if (!match) {
      return {
          name: parsedItem.name,
          baseLevel: parsedItem.baseLevel,
          enchantValue: parsedItem.enchantValue,
          foundInJson: false
      };
  }

  // Berechne die Effekte
  const itemEffects = calculateItemEffects(match.data, parsedItem.baseLevel, parsedItem.enchantValue);

  return {
      originalText: parsedItem.originalText,
      normalizedName: parsedItem.name,
      nameInJson: match.key,
      foundInJson: true,
      type: match.data.type ?? 'unknown',
      slot: match.data.slot ?? 'unknown',
      baseLevel: parsedItem.baseLevel,
      enchantValue: parsedItem.enchantValue,
      tattooLevel: parsedItem.tattooLevel,
      calculatedEffects: itemEffects
  };
}

// Export utils
window.CBUtils = {
  statAbbrev,
  normalizeItemName,
  safeParse,
  getSidebarFrame,
  parseEquippedItemText,
  searchItemInJSON,
  calculateAC,
  calculateItemEffects,
  buildEquippedItem
};